/***************************************************************************
 *                                                                         *
 *    NoMoRe++                                                             *
 *                                                                         *
 *    Copyright (C) 2003-2005 NoMoRe Developing Group                      *
 *                                                                         * 
 *    For more information, see http://www.cs.uni-potsdam.de/nomore/       *
 *    or email to nomore-dg@cs.uni-potsdam.de                              *
 *                                                                         *
 *                                                                         *
 *    This program is free software; you can redistribute it and/or        *
 *    modify it under the terms of the GNU General Public License          *
 *    as published by the Free Software Foundation; either version 2       *
 *    of the License, or (at your option) any later version.               *
 *                                                                         *
 *    This program is distributed in the hope that it will be useful,      *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *    GNU General Public License for more details.                         *
 *                                                                         *
 *    You should have received a copy of the GNU General Public License    *
 *                                                                         * 
 ***************************************************************************/

#include <heuristics/neg_arcs_count.h>
#include <head_node.h>
#include <body_node.h>
namespace NS_NOMORE {

///////////////////////////////////////////////////////////////////////////////
template <>
const char* getName(NegativeWeightArcsCount*) {
  return "NegArcs";
}

template <>
Heuristic* createHeuristic(NegativeWeightArcsCount*) {
  return new NegativeWeightArcsCount();
}
///////////////////////////////////////////////////////////////////////////////


NegativeWeightArcsCountStrat::ValueType NegativeWeightArcsCountStrat::Evaluate::operator()(Node* n, double tf) const {
  assert(0 != n);

  if (n->getType() == NodeType::body_node) {
    BodyNode* bn = static_cast<BodyNode*>(n);
    ValueType res(0, (long)(bn->countNegBodyAtoms() * tf));
    for(BodyNode::HeadNodeIterator it = bn->successorsBegin(); it != bn->successorsEnd(); ++it)
      res.negSuccessors_ += (*it)->countOneSuccessors();
    res.negSuccessors_ = (long)(res.negSuccessors_ * tf);
    return res;
  }
  else {
    assert(n->getType() == NodeType::head_node);
    HeadNode* hn = static_cast<HeadNode*>(n);
    ValueType res((long)(hn->countOneSuccessors() * tf), 0);
    for(HeadNode::BodyNodeIterator it = hn->predecessorsBegin(); it != hn->predecessorsEnd(); ++it)
      res.negPredecessors_ += (*it)->countNegBodyAtoms();
    res.negPredecessors_ = (long)(res.negPredecessors_ * tf);
    return res;
  }
}

}
